#pragma region Copyright 2012 IPETRONIK GmbH & Co.KG
//
// All rights are reserved. Reproduction or transmission in whole or in part, in
// any form or by any means, electronic, mechanical or otherwise, is prohibited
// without the prior written consent of the copyright owner.
//
// KGI
//
#pragma endregion
/// <summary>
/// ExtensionDllTypes.h
/// </summary>
#pragma once

/// <summary>
/// Value definitions
/// </summary>
#define NO_SYSTEM 0
#define NO_PACKET 0

/// <summary>
/// define
/// </summary>
#define _WIN32_WINNT 0x0501
#define WIN32_LEAN_AND_MEAN

/// <summary>
/// include
/// </summary>
#include <windows.h>
#include <float.h>

/// <summary>
/// Typ-Definitionen
/// </summary>
typedef int					BOOL;
typedef unsigned char		UCHAR;
typedef unsigned short		USHORT;
typedef signed long			LONG;
typedef unsigned long		ULONG;
typedef wchar_t				WCHAR;
typedef HRESULT				HRESULT;

#ifndef MAX_WCHAR_TRACE
// 225 = 260 - 29 - 6
// for security decrement by one!
// any message being longer causes a program crash.
// 260: Maximum lenght set in IPEmotion for the whole log message on a line
// 29:  Number of characters needed for timestamp, separators, severity value (1,2 or 3) and the trailing \r\n in a line
// 6:   length of the MAL_NAME string L"Serial"
	#define MAX_WCHAR_TRACE	224
#endif

#ifndef MAX_WCHAR
	#define MAX_WCHAR			MAX_WCHAR_TRACE
#endif

/// <summary>
/// Definitionen
/// </summary>
#define EXTENSION_API			__stdcall
#define EXTENSION_EXPORT		__declspec(dllexport)

#define EXTENSION_API_VERSION	1

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Define (base) structure for passing system parameters to the extension dll and back
///
/// Don't access these parameters directly due to compatibility issues.
/// Parameters should be accessed by special functions defined in 'ExtensionAuxiliaryFunctions.cpp/h handeling
/// backward compatibility!:
/// </summary>
#pragma pack(push, 1)
#define SYSTEM_PARAMETER_VERSION 1
struct TSystemParameter
{
	UINT  uiStructVersion; // SYSTEM_PARAMETER_VERSION - to be incremented when struct is modified
	ULONG ulIdentifier;
	BOOL   bActive;
	// only APPEND new values, NEVER CHANGE exsisting struct!
};
#pragma pack(pop)

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Define (base) structure for passing packet parameters to the extension dll and back
///
/// Don't access these parameters directly due to compatibility issues.
/// Parameters should be accessed by special functions defined in 'ExtensionAuxiliaryFunctions.cpp/h handeling
/// backward compatibility!:
/// </summary>
#pragma pack(push, 1)
#define PACKET_PARAMETER_VERSION 1
struct TPacketParameter
{
	UINT  uiStructVersion;// PACKET_PARAMETER_VERSION - to be incremented when struct is modified
	ULONG ulIdentifier;
	BOOL   bActive;
	// only APPEND new values, NEVER CHANGE exsisting struct!
};
#pragma pack(pop)


/// <summary>
/// Callback function type definitions
/// </summary>
typedef HRESULT (EXTENSION_API * PFN_EXT_SET_PACKET_DATA_FLOAT64_FUNC)(	ULONG ulSystemIdentifier,
																		ULONG ulPacketIdentifier,
																		double dDataFloat64);
typedef HRESULT (EXTENSION_API * PFN_EXT_SEND_DATA_FUNC)(	ULONG ulSystemIdentifier,
															LONG lDataSize,
															UCHAR* paucData);
typedef LONG    (EXTENSION_API * PFN_EXT_READ_INPUT_BUFFER)(ULONG ulSystemIdentifier,
															LONG lBufferSize,
															UCHAR* paucBuffer,
															LONG* plReadBytesCount);
typedef HRESULT (EXTENSION_API * PFN_EXT_RESET_INPUT_BUFFER)(ULONG ulSystemIdentifier);
typedef HRESULT (EXTENSION_API * PFN_EXT_TRACE_MESSAGE)(WCHAR *wszTrace);

/// <summary>
/// Dll function type definitions
/// </summary>
typedef HRESULT (EXTENSION_API * ExtSetDiagnosticsVersionProc)(BOOL);
typedef ULONG   (EXTENSION_API * ExtGetApiVersionProc)();

typedef HRESULT (EXTENSION_API * ExtInitDriverProc)(WCHAR* pwszErrorMessage);
typedef HRESULT (EXTENSION_API * ExtExitDriverProc)(WCHAR* pwszErrorMessage);

typedef HRESULT (EXTENSION_API * ExtSetCallbacksProc)(	PFN_EXT_SET_PACKET_DATA_FLOAT64_FUNC pfnDataFloat64,
														PFN_EXT_SEND_DATA_FUNC               pfnSendData,
														PFN_EXT_READ_INPUT_BUFFER            pfnReadInputBuffer,
														PFN_EXT_RESET_INPUT_BUFFER           pfnResetInputBuffer,
														PFN_EXT_TRACE_MESSAGE                pfnTraceInformation,
														PFN_EXT_TRACE_MESSAGE                pfnTraceWarning,
														PFN_EXT_TRACE_MESSAGE                pfnTraceError);

typedef HRESULT (EXTENSION_API * ExtPrepareAcquisitionProc)(	ULONG ulSystemIdentifier);
typedef HRESULT (EXTENSION_API * ExtPrepareAcquisitionV2Proc)(	ULONG              ulSystemIdentifier,
																TSystemParameter*   pSystemParameter,
																USHORT             usPacketParameterCount,
																TPacketParameter** ppPacketParameter);
typedef HRESULT (EXTENSION_API * ExtDoPreAcquisitionTimerProc)(	ULONG ulSystemIdentifier);
typedef HRESULT (EXTENSION_API * ExtDoPostAcquisitionTimerProc)(ULONG ulSystemIdentifier);
typedef HRESULT (EXTENSION_API * ExtFinishAcquisitionProc)(		ULONG ulSystemIdentifier);

typedef HRESULT (EXTENSION_API * ExtSetOutputDataFloat64Proc)(	ULONG   ulSystemIdentifier,
																ULONG   ulPacketIdentifier,
																double *pdDataFloat64,
																BOOL   *pbTransferValueOut);
