#pragma region Copyright 2012 IPETRONIK GmbH & Co.KG
//
// All rights are reserved. Reproduction or transmission in whole or in part, in
// any form or by any means, electronic, mechanical or otherwise, is prohibited
// without the prior written consent of the copyright owner.
//
// KGI
//
#pragma endregion

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// include
/// </summary>
#include "Extension.h"

#define _STUB_SYNTHETIC_SINUS_INPUT

#ifndef STUB_SYNTHETIC_SINUS_INPUT
#define RECEIVE_DATA_VIA_COM_EXAMPLE
#endif

#ifdef STUB_SYNTHETIC_SINUS_INPUT
#include <mmsystem.h>             // needed for function timeGetTime() used in example code
#endif
///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// lib
/// </summary>
#pragma comment(lib, "Version.lib")

#ifdef STUB_SYNTHETIC_SINUS_INPUT
#pragma comment(lib, "Winmm.lib") // needed for function timeGetTime() used in example code
#endif
///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Global member: callback functions
/// </summary>
PFN_EXT_SET_PACKET_DATA_FLOAT64_FUNC g_pfnDataFloat64      = NULL; // transfer packet data of double type to the plugin
PFN_EXT_SEND_DATA_FUNC               g_pfnSendData         = NULL; // send data via the systems COM port
PFN_EXT_READ_INPUT_BUFFER            g_pfnReadInputBuffer  = NULL; // read new data received via the
                                                                   // systems COM port since last call of g_pfnReadInputBuffer
PFN_EXT_RESET_INPUT_BUFFER           g_pfnResetInputBuffer = NULL; // reset the input buffer of the systems COM port

PFN_EXT_TRACE_MESSAGE                g_pfnTraceInformation = NULL; // Set a information message to the trace log. Message is
                                                                   // a null terminated WCHAR string with maximum length of MAX_WCHAR (currently 256)
PFN_EXT_TRACE_MESSAGE                g_pfnTraceWarning     = NULL; // Set a warning message to the trace log. Message is
                                                                   // a null terminated WCHAR string with maximum length of MAX_WCHAR (currently 256)
PFN_EXT_TRACE_MESSAGE                g_pfnTraceError       = NULL; // Set a error message to the trace log. Message is
                                                                   // a null terminated WCHAR string with maximum length of MAX_WCHAR (currently 256)

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Called once as 1st method after the PlugIn has imported the functions (exported by this  the driver is being initialized. This function is called once at the end of the PlugIn function "InitDriver".
/// InitDriver is called directly after SetCallback.
/// </summary>
/// <param name="isDiagnosticsVersion">
///   FALSE (default) when PlugIn is started in normal mode.
///   TRUE if the PlugIn is started in diagnostics mode. The dll should store this value and evaluate it in the other functions for tracing additional messages
///        that would normally not logged due to performance issues.
/// </param>
HRESULT EXTENSION_API SetDiagnosticsVersion(BOOL isDiagnosticsVersion)
{
	HRESULT RetVal = NO_ERROR;

	// sets global variable indicating that additional messages are logged
	// use function 'IsDiagnosticsVersion()' in code
	DoSetDiagnosticsVersion(isDiagnosticsVersion);

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Called when the driver is being initialized. This function is called once at the end of the PlugIn function "InitDriver".
/// InitDriver is called directly after SetCallback.
/// </summary>
/// <param name="pwszErrorMessage">
///   reserved for future use - 
///   pointer to buffer for an error message, propagated to
///   IPEmotion when function returns E_FAIL
/// </param>
HRESULT EXTENSION_API InitDriver(WCHAR* pwszErrorMessage)
{
	HRESULT RetVal = NO_ERROR;

	// place optional initializing code here

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Called when exiting driver. This function is called once at the beginning of the PlugIn function "ExitDriver".
/// </summary>
/// <param name="pwszErrorMessage">
///   reserved for future use - 
///   pointer to buffer for an error message, propagated to
///   IPEmotion when function returns E_FAIL
/// </param>
HRESULT EXTENSION_API ExitDriver(WCHAR* pwszErrorMessage)
{
	HRESULT RetVal = NO_ERROR;

	// Place optional uninitializing code here. E.g.:
	// - free memory previously allocated with new or malloc
	// - release resources if any have been reserved
	// - close any opened files
	// - close any opened drivers
	// - ...
	// If this is not done (if needed) the dll might not unload when IPEmotioin is being closed
	// such that it appears that IPEmotion or the Plugin hangs up.

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Assignment of callback functions. This function is called once at the end of the PlugIn function "InitDriver".
/// SetCallbacks is called just before InitDriver.
/// </summary>
HRESULT EXTENSION_API SetCallbacks(PFN_EXT_SET_PACKET_DATA_FLOAT64_FUNC pfnDataFloat64,
                                   PFN_EXT_SEND_DATA_FUNC               pfnSendData,
                                   PFN_EXT_READ_INPUT_BUFFER            pfnReadInputBuffer,
                                   PFN_EXT_RESET_INPUT_BUFFER           pfnResetInputBuffer,
                                   PFN_EXT_TRACE_MESSAGE                pfnTraceInformation,
                                   PFN_EXT_TRACE_MESSAGE                pfnTraceWarning,
                                   PFN_EXT_TRACE_MESSAGE                pfnTraceError)
{
	HRESULT RetVal = NO_ERROR;

	// Set callback functions
	// This call is mandatory.
	RetVal = DoSetCallbacks(pfnDataFloat64, pfnSendData, pfnReadInputBuffer, pfnResetInputBuffer,
							pfnTraceInformation, pfnTraceWarning, pfnTraceError);

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// ATTENTION: If 'PrepareAcquisitionV2()' is exported, the function '"PrepareAcquisition()' is NOT called!
/// Measurement preparation. This function is called once for every System in the PlugIn function 'PrepareAcquisition'.
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
/// </param>
HRESULT EXTENSION_API PrepareAcquisition(ULONG ulSystemIdentifier)
{
	HRESULT RetVal = NO_ERROR;

	// do some prepartion if needed, eg send some start or configuration signals

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// ATTENTION: If 'PrepareAcquisitionV2()' is exported, the function '"PrepareAcquisition()' is NOT called!
/// Measurement preparation. This function is called once for every System in the PlugIn function "PrepareAcquisition".
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
/// </param>
/// <param name="pSystemParameter">
///   Pointer to a structure containing system parameters.
///   Don't access these parameters directly due to compatibility issues.
///   Parameters should be accessed by special functions defined in 'ExtensionAuxiliaryFunctions.cpp/h handeling
///   backward compatibility!:
/// </param>
/// <param name="usPacketParameterCount">
///   Number of elements in the array ppPacketParameter
/// </param>
/// <param name="ppPacketParameter">
///   Array with pointers to structures containing packet parameters (connected under the given system).
///   Don't access these parameters directly due to compatibility issues.
///   Parameters should be accessed by special functions defined in 'ExtensionAuxiliaryFunctions.cpp/h handeling
///   backward compatibility!:
/// </param>
HRESULT EXTENSION_API PrepareAcquisitionV2(ULONG ulSystemIdentifier,      TSystemParameter*   pSystemParameter,
                                           USHORT usPacketParameterCount, TPacketParameter** ppPacketParameter)
{
	HRESULT RetVal = NO_ERROR;

	USHORT  usIndex = 0;

	BOOL   bActive           = false;
	ULONG ulPacketIdentifier = 0;

	// do some prepartion if needed, eg send some start or configuration signals
	// this is just example code demonstrating the principal usage of the function parameters
	if(pSystemParameter != NULL)
	{
		if(SUCCEEDED(GetActive(pSystemParameter, &bActive)))
		{
			if(ppPacketParameter != NULL)
			{
				for(usIndex = 0; usIndex < usPacketParameterCount; usIndex++)
				{
					if(ppPacketParameter[usIndex] != NULL)
					{
						if(SUCCEEDED(GetActive(ppPacketParameter[usIndex], &bActive)))
						{
							if(bActive)
							{
								if(SUCCEEDED(GetIdentifier(ppPacketParameter[usIndex], &ulPacketIdentifier)))
								{
									// TODO: place code here for the given packet identifier under the given system identifier
								}
							}
						}
					}
				}
			}
		}
	}

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Function called on the beginning of measurement timer (cycle).
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
/// </param>
HRESULT EXTENSION_API DoPreAcquisitionTimer(ULONG ulSystemIdentifier)
{
	HRESULT RetVal = NO_ERROR;

	// send some commands via COM port if needed

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Function called on the end of measurement timer (cycle)
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
/// </param>
HRESULT EXTENSION_API DoPostAcquisitionTimer(ULONG ulSystemIdentifier)
{
	HRESULT RetVal = NO_ERROR;

	// read data from COM port until data is complete
	// parse read data for packet values
	// transmit packet data to plugin via data-callback g_pfnDataFloat64

	ULONG ulPacketIdentifier = 0;
	double dDataFloat64 = -1;

#ifdef STUB_SYNTHETIC_SINUS_INPUT
	DWORD dwTime = timeGetTime();
	dDataFloat64 = sin((double) dwTime);

	// Packet Identifier is 1
	ulPacketIdentifier = 1;
	g_pfnDataFloat64(ulSystemIdentifier, ulPacketIdentifier, dDataFloat64);
#endif

#ifdef RECEIVE_DATA_VIA_COM_EXAMPLE
	UCHAR readBuffer[256];
	memset(readBuffer, 0, sizeof(readBuffer));
	LONG bytesReadCount = 0;

	//receive
	int i   = 0;
	int j   = 0;
	int pos = 0; // current start index in the input buffer
	for(i = 0; i < 10; i++) // max wait time is 50ms
	{
		Sleep(5);

		RetVal = g_pfnReadInputBuffer(ulSystemIdentifier, 255-pos, &(readBuffer[pos]), &bytesReadCount);
		if(RetVal == E_FAIL)
			break;

		if(bytesReadCount > 0)
		{
			// test if we are finished - each answer ends with 0x0D 0x0A
			if(readBuffer[pos + bytesReadCount - 1] == 0x0A)
			{
				// seek start position of last string in buffer
				for(j = pos + bytesReadCount - 1; j > 0; j--)
				{
					if(readBuffer[j-1] == 0x0A)
						break;
				}
				// now j should be not negative!

				// parse value
				if(sscanf_s((char*)&(readBuffer[j]), "PacketId %d - Out: %lf\n", &ulPacketIdentifier, &dDataFloat64) == 2)
				{
					g_pfnDataFloat64(ulSystemIdentifier, ulPacketIdentifier, dDataFloat64);
					break;
				}
			}

			// set next start position
			pos += bytesReadCount;
		}

		// if pos == 0, then no data has been sent in the meantime and
		// it has not to be waited for completition
		if(pos == 0)
			break;
	}
#endif

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Measurement finalization.This function is called once for every System in the PlugIn function "FinishAcquisition".
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
/// </param>
HRESULT EXTENSION_API FinishAcquisition(ULONG ulSystemIdentifier)
{
	HRESULT RetVal = NO_ERROR;

	// do some finalizing if needed, eg send som stop signals free memory / resources if needed

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Measurement finalization.This function is called once for every System in the PlugIn function "SetOutputDataFloat64".
/// </summary>
/// <param name="ulSystemIdentifier">
///   Identifier, defined on the configuration tab page "Identifer" in IPEmotion
///   This identifier can be used to distinguish different "Serial" systems, if
///   more than one are defined in IPEmotion.
///   A value of 0 is invlaid!
/// </param>
/// <param name="ulPacketIdentifier">
///   Identifier, defined on the channel configuration tab page "Identifer" in IPEmotion.
///   This identifier can be used to distinguish different output channels, if
///   more than one are defined in IPEmotion.
///   A value of 0 is invlaid!
/// </param>
/// <param name="pdDataFloat64">
///   Pointer to the output value to be set.
///   IMPORTANT: The pointer is used to transfer a modified value to the PlugIn. Depending on the value set to pbTransferValueOut
///              the plugin will or will not propagate the returned value to IPEmotion
/// </param>
/// <param name="pbTransferValueOut">
///   pointer to BOOL variable which is read by the calling function. It can be set by the function to tell the PlugIn if the extension dll
///              has already transmitted the current output value to IPEmotion or if the PlugIn should do it.
///   IMPORTANT: TRUE:  The PlugIn will transfer the returned value of *pdDataFloat64 to IPEmotion
///              FALSE: The PlugIn should not send the value to IPEmotion
/// </param>
HRESULT EXTENSION_API SetOutputDataFloat64(ULONG ulSystemIdentifier, ULONG ulPacketIdentifier, double *pdDataFloat64, BOOL *pbTransferValueOut)
{
	// This EXAMPLE shows how to send data via an output channel.
	// The provided double Value is simply sent via the serial interface and written back to
	// IPEmotion by the corresponding data callback.

	HRESULT RetVal = NO_ERROR;

	UCHAR dataBuffer[256];

	memset(dataBuffer, 0, sizeof(dataBuffer));

	// corresponding to the communication protocol and depending on the packet identifier / channel the send data
	// has to be provided here.
	*pdDataFloat64 = *pdDataFloat64 + 0.1;
	sprintf_s((char*)dataBuffer, sizeof(dataBuffer), "PacketId %d - Out: %lf\n", ulPacketIdentifier, *pdDataFloat64);

	// send the output buffer
	LONG BufferSize = std::strlen((char*)dataBuffer);
	g_pfnSendData(ulSystemIdentifier, BufferSize, dataBuffer);

	// Here one MUST wait for the answer if any (keep in mind to not interfer with the parallel process of reading data).

	//// Now write back the true output value to IPEmotion such that the value will be displayed correctly.
	//// In this example the value is plainly written back but it could be different from the provided one.
	//// Mind to use the provided system and packet identifiers

	// depending on the extension dll, uncomment the next 2 code lines to let the extension dll transfer the current
	// output values to IPEmotion
	//g_pfnDataFloat64(ulSystemIdentifier, ulPacketIdentifier, *pdDataFloat64);
	//*pbTransferValueOut = FALSE;

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////

#ifdef __cplusplus
}
#endif
