#pragma region Copyright 2012 IPETRONIK GmbH & Co.KG
//
// All rights are reserved. Reproduction or transmission in whole or in part, in
// any form or by any means, electronic, mechanical or otherwise, is prohibited
// without the prior written consent of the copyright owner.
//
// KGI
//
#pragma endregion


///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// ATTENTION
/// ATTENTION DO NOT MODIFY THIS FILE BY HAND!
/// ATTENTION
/// </summary>
///////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// ExtensionAuxiliaryFunctions.cpp: defines some auxiliary functions to keep mandatory code
///                                  out of Extension.cpp where the individual code will be placed
/// </summary>

///////////////////////////////////////////////////////////////////////////////////////////
/// 
/// include
/// 
#include "Extension.h"

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Global member: Dll related data (defined in Serial.Acquisition.Extension.cpp)
/// </summary>
BOOL g_bIsDiagnosticsVersion = FALSE;

#ifdef __cplusplus
extern "C" {
#endif
///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Export function with definition of API-Version
/// </summary>
ULONG GetApiVersion()
{
	return EXTENSION_API_VERSION;
}
#ifdef __cplusplus
}
#endif

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Function indicating that additional messages are to be logged (if any)
/// </summary>
bool IsDiagnosticsVersion()
{
	return (g_bIsDiagnosticsVersion == TRUE);
};

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Called once as 1st method after the PlugIn has imported the functions (exported by this  the driver is being initialized. This function is called once at the end of the PlugIn function "InitDriver".
/// InitDriver is called directly after SetCallback.
/// </summary>
/// <param name="isDiagnosticsVersion">
///   FALSE (default) when PlugIn is started in normal mode.
///   TRUE if the PlugIn is started in diagnostics mode. The dll should store this value and evaluate it in the other functions for tracing additional messages
///        that would normally not logged due to performance issues.
/// </param>
HRESULT DoSetDiagnosticsVersion(BOOL isDiagnosticsVersion)
{
	HRESULT RetVal = NO_ERROR;

	g_bIsDiagnosticsVersion = isDiagnosticsVersion;

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Assignment of callback functions
/// </summary>
HRESULT DoSetCallbacks(	PFN_EXT_SET_PACKET_DATA_FLOAT64_FUNC pfnDataFloat64,
                        PFN_EXT_SEND_DATA_FUNC               pfnSendData,
                        PFN_EXT_READ_INPUT_BUFFER            pfnReadInputBuffer,
                        PFN_EXT_RESET_INPUT_BUFFER           pfnResetInputBuffer,
                        PFN_EXT_TRACE_MESSAGE                pfnTraceInformation,
                        PFN_EXT_TRACE_MESSAGE                pfnTraceWarning,
                        PFN_EXT_TRACE_MESSAGE                pfnTraceError)
{
	HRESULT RetVal = NO_ERROR;

	// Set callback functions for:

	// - transmitting packet / channel data to the plugin
	g_pfnDataFloat64      = pfnDataFloat64;

	// - sending data via the systems COM port
	g_pfnSendData         = pfnSendData;

	// - reading last received input data
	g_pfnReadInputBuffer  = pfnReadInputBuffer;

	// - resetting input data buffer
	g_pfnResetInputBuffer = pfnResetInputBuffer;

	// - setting an information message to the trace log
	g_pfnTraceInformation = pfnTraceInformation;

	// - setting an warning message to the trace log
	g_pfnTraceWarning     = pfnTraceWarning;

	// - setting an error message to the trace log
	g_pfnTraceError       = pfnTraceError;

	return RetVal;
}

///////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Get the active flag of the given structure
/// </summary>
HRESULT GetActive(TSystemParameter *pParameter, BOOL *pbActive)
{
	HRESULT hRes = E_FAIL;

	if( (pParameter != NULL) && (pbActive != NULL) )
	{
		// 1st verify parameter version - only set parameter if included in version
		if(pParameter->uiStructVersion >= 1)
		{
			*pbActive = pParameter->bActive;

			hRes = NO_ERROR;
		}
	}

	return hRes;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Get the active flag of the given structure
/// </summary>
HRESULT GetIdentifier(TSystemParameter *pParameter, ULONG *pulIdentifier)
{
	HRESULT hRes = E_FAIL;

	if( (pParameter != NULL) && (pulIdentifier != NULL) )
	{
		// 1st verify parameter version - only set parameter if included in version
		if(pParameter->uiStructVersion >= 1)
		{
			*pulIdentifier = pParameter->ulIdentifier;

			hRes = NO_ERROR;
		}
	}

	return hRes;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Get the active flag of the given structure
/// </summary>
HRESULT GetActive(TPacketParameter *pParameter, BOOL *pbActive)
{
	HRESULT hRes = E_FAIL;

	if( (pParameter != NULL) && (pbActive != NULL) )
	{
		// 1st verify parameter version - only set parameter if included in version
		if(pParameter->uiStructVersion >= 1)
		{
			*pbActive = pParameter->bActive;

			hRes = NO_ERROR;
		}
	}

	return hRes;
}

///////////////////////////////////////////////////////////////////////////////////////////
/// <summary>
/// Get the active flag of the given structure
/// </summary>
HRESULT GetIdentifier(TPacketParameter *pParameter, ULONG *pulIdentifier)
{
	HRESULT hRes = E_FAIL;

	if( (pParameter != NULL) && (pulIdentifier != NULL) )
	{
		// 1st verify parameter version - only set parameter if included in version
		if(pParameter->uiStructVersion >= 1)
		{
			*pulIdentifier = pParameter->ulIdentifier;

			hRes = NO_ERROR;
		}
	}

	return hRes;
}
